import logging
import numpy as np


class DataHandler:
    def __init__(self, var, t0, cell_slice, prune_offset=None):
        self.var = var
        self.cell_slice = cell_slice
        shape = {dim: chunk_size for dim, chunk_size in zip(var.attrs["_ARRAY_DIMENSIONS"], var.shape)}
        mi, ma, st = cell_slice.indices(shape["cell"])
        assert st == 1, "non-contiguous cell slices are not supported"
        self.t = t0
        self.prune_offset = prune_offset
        self.t_flushed = self.t
        chunks = {dim: chunk_size for dim, chunk_size in zip(var.attrs["_ARRAY_DIMENSIONS"], var.chunks)}
        self.time_chunk_size = chunks["time"]
        self.buf = np.empty([self.time_chunk_size, *var.shape[1:-1], ma-mi], dtype=var.dtype)

    def flush(self):
        if self.prune_offset is not None:
            l = max(0, self.t_flushed-self.prune_offset)
            r = max(0, self.t-self.prune_offset)
            logging.info(f"pruning {self.var.name} for timesteps {l}:{r}")
            self.var[l:r, ..., self.cell_slice] = self.var.fill_value
        logging.info(f"writing {self.var.name} for timesteps {self.t_flushed}:{self.t}")
        self.var[self.t_flushed:self.t, ..., self.cell_slice] = self.buf[-(self.t-self.t_flushed):, ...]
        self.t_flushed = self.t

    def __call__(self, event):
        self.buf[self.t % self.time_chunk_size, ...] = event.data
        logging.info(f"received {self.var.name} at {self.t % self.time_chunk_size+1}/{self.time_chunk_size}")
        self.t += 1
        if self.t % self.time_chunk_size == 0:  # chunk complete
            self.flush()
