import asyncio
from aiohttp import web
from threading import Thread


def expose_store_over_http(store, host, port):

    def _serve(request):
        try:
            key = request.match_info["key"]
            data = store[key]
        except KeyError:
            return web.Response(status=404, reason="Data not found!")
        return web.Response(body=data)

    def _run_server(runner, host, port):
        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)
        loop.run_until_complete(runner.setup())
        site = web.TCPSite(runner, host, port)
        loop.run_until_complete(site.start())
        loop.run_forever()

    app = web.Application()
    app.add_routes([web.get('/{key:.*}', _serve)])
    runner = web.AppRunner(app)
    t = Thread(target=_run_server, args=(runner, host, port),
               daemon=True)
    t.start()


def write_nginx_config(distributed_data_vars, port, rank):
    import platform
    from regex_engine import generator

    with open("backend_map.conf", "w") as f:
        f.write("map $request_uri $backend {\n")
        for r, data_vars in enumerate(distributed_data_vars):
            for group, vgroup, chunk_slice, data_vars in data_vars:
                for v in data_vars:
                    if chunk_slice == slice(None):
                        cell_regex = r"\d+"
                    else:
                        cell_chunk_size = dict(zip(v.attrs["_ARRAY_DIMENSIONS"], v.chunks))["cell"]
                        chunk_range = (chunk_slice.start//cell_chunk_size, chunk_slice.stop//cell_chunk_size)
                        cell_regex = generator().numerical_range(*chunk_range)[1:-1]
                    f.write(f"  \"~^/{group}/{v.name}/\\d+\\.\\d+\\.{cell_regex}\" loco_backend{r};\n")
        f.write("}\n\n")
        for i in range(len(distributed_data_vars)):
            f.write(f"include \"loco_backend{i}.conf\";\n")

    with open(f"loco_backend{rank}.conf", "w") as f:
        f.write(f"""
upstream loco_backend{rank} {{
    server {platform.node()}:{port};
}}
""")
