#ifndef COYOTE_COYOTE_HPP
#define COYOTE_COYOTE_HPP

#include <string>
#include <functional>
#include <memory>
#include <optional>
#include <span>
#include <mpi.h>
#include <map>

namespace coyote {

  struct Event{
    std::vector<double> data;
    int collection_size;
    size_t point_size;
  };

  class Coyote{
    class Impl;
    std::unique_ptr<Impl> pImpl;
  public:
    Coyote(Coyote&&);
    Coyote(const Coyote&) = delete;
    Coyote(const std::string& name);
    Coyote& operator=(Coyote&&);
    Coyote& operator=(const Coyote&) = delete;
    ~Coyote();

    void add_field(
                   const std::string& source_comp,
                   const std::string& source_grid,
                   const std::string& field_name,
                   const std::function<void(const Event& event)>& eventHandler,
                   std::optional<std::string> iso_dt = std::nullopt,
                   std::optional<int> collection_size = std::nullopt,
                   std::string interpolation_stack_yaml = "[nnn]",
                   int yac_time_reduction = 0,
                   std::vector<double> frac_mask = {},
                   bool pass_through = true,
                   int src_lag = 1);

    MPI_Comm get_mpi_comm() const;

    // grids
    // yac wrapper
    void def_grid(std::span<const int> num_vertices_per_cell,
                  std::span<const double> x_vertices,
                  std::span<const double> y_vertices,
                  std::span<const int> cell_to_vertex,
                  std::optional<std::span<const double>> x_cells = std::nullopt,
                  std::optional<std::span<const double>> y_cells = std::nullopt,
                  std::optional<std::span<const int>> global_cell_idx = std::nullopt);

    std::vector<int> def_healpix_grid(int order,
                                      std::optional<std::tuple<size_t, size_t>> cellidx_range);
    

    // functions to get the necessary info on the available variables
    int get_field_collection_size(const std::string& source_comp,
                                  const std::string& source_grid,
                                  const std::string& field_name) const;
    std::string get_field_timestep(const std::string& source_comp,
                                  const std::string& source_grid,
                                  const std::string& field_name) const;
    std::string get_field_metadata(const std::string& source_comp,
                                  const std::string& source_grid,
                                  const std::string& field_name) const;
    std::string get_start_datetime() const;
    std::string get_end_datetime() const;

    std::map<std::string, std::map<std::string, std::vector<std::string>>> get_all_comp_grid_fields() const;
  };

  void coyote_init(std::string group_name = "");
  MPI_Comm coyote_group_comm();
  std::string coyote_start_datetime();
  std::string coyote_end_datetime();
  void coyote_run();

}
#endif
