#ifndef COYOTE_COYOTEENV_HPP
#define COYOTE_COYOTEENV_HPP

#include <thread>

#include "tsqueue.hpp"
#include "event.hpp"

namespace coyote {

  using EventQueue = TSQueue<std::shared_ptr<FieldBuffer>>;

  class CoyoteEnv{
    friend void coyote_init(std::string group_name);
  public:
    CoyoteEnv(std::string group_name = "");

    static CoyoteEnv& getInstance();

    MPI_Comm get_group_comm() const;
    int get_yac_instance_id() const;
    std::string start_datetime();
    std::string end_datetime();
    void add_field(std::unique_ptr<Field> field);
    void ensure_def_comps();
    void ensure_sync_def();
    void ensure_enddef();
    void run();
  private:
    static std::unique_ptr<CoyoteEnv> _instance;
    int yac_instance_id;
    MPI_Comm group_comm;
    bool state_def_comps, state_sync_def, state_enddef;
    std::thread put_thread, get_thread;
    EventQueue _main_queue, _put_queue;
    std::vector<std::unique_ptr<Field>> _fields;
    bool _finalize_mpi = false;
  };

}

#endif
