#ifndef COYOTE_QUEUE_HPP
#define COYOTE_QUEUE_HPP

#include <queue>
#include <mutex>
#include <condition_variable>
#include <optional>
#include <functional>
#include <thread>

class QueueTerminated {};

template<class T>
class TSQueue{

public:
  TSQueue()
    : q()
    , m()
    , c()
  {}

  ~TSQueue() {}

  void enqueue(T t)
  {
    std::lock_guard<std::mutex> lock(m);
    q.push(t);
    c.notify_one();
  }

  void end()
  {
    std::lock_guard<std::mutex> lock(m);
    q.push(std::nullopt);
    c.notify_all();
  }

  T dequeue()
  {
    std::unique_lock<std::mutex> lock(m);
    c.wait( lock, [&]{ return !q.empty(); } );
    std::optional<T> val = q.front();
    if(!val){
      throw QueueTerminated();
    }
    q.pop();
    return val.value();
  }

  bool terminated(){
    std::unique_lock<std::mutex> lock(m);
    if(q.empty())
      return false;
    std::optional<T> val = q.front();
    return !val.has_value;
  }


private:
  std::queue<std::optional<T>> q;
  mutable std::mutex m;
  std::condition_variable c;

};

#endif
